

#ifndef _MY_THERADS_H_
#define _MY_THERADS_H_



#include <setjmp.h>

//#define ST_TEST 1
//#define CC_TEST 1
//#define LA_TEST 1
//#define FD_TEST 1


typedef unsigned long long int long_t;

#define CONST_PTN 0
#define EXP_PTN 1
#define HALF_SIN_PTN 2
#define SIN_PTN 3
#define BASE_PTN 0.875
#define MAX_PTN 5




#define NUM_THREADS (numThreads-1)
#define DELAY_USEC 200
#define CCWIN 3
#define MAX_ERR_ITEMS 30
#define MAX_INTEGRATION_ITEM 5  /*---- should be less than MAX_ERR_ITEMS ----*/



/*  sync switching or asyn switching  */
#ifndef SYNC
#define SYNC 1
#endif



/*
 * number of freq. levels.  If changed, need to change
 * the freq. setting table in my_threads.c and 
 * the my_set_freq() function in dpm405lp.c, too.
 */
#define NUM_FREQ 5

#define DVS_ST 0
#define DVS_CC 1
#define DVS_LA 2
#define DVS_FD 3
#define DVS_AGR 4
#define DVS_FIXED 6
#define DVS_ORG_FD 8
#define DVS_OTE 5
#define DVS_ORG_LA 7

#define	NOERROR	0
#define	MAXTHREADS	1
#define MALLOCERROR	2
#define CLONEERROR	3
#define	INTHREAD	4
#define SIGNALERROR	5

#ifdef DEBUG
#define mytrace(args...) do {printf("TRACE:");printf(args);} while (0)
#else
#define mytrace(args...) do {} while (0)
#endif


/* define bool, true and false */
#define bool char
#define true 1
#define false 0
#define TRUE 1
#define FALSE 0

#define MAX_FREQ_INDEX (NUM_FREQ*2-1)  /* Santhosh: Changed for the addition of new OPs */


#define P_P 0.8
#define P_I 0.01
#define P_D 0.9

/* The maximum number of threads that can be active at once. */
#define MAX_THREADS 31

/* The size of the stack for each thread. */
#define THREAD_STACK 1024*1024



struct avgErrType{
  long_t  err[MAX_ERR_ITEMS];
  int head;
};

typedef struct
{
  sigjmp_buf context;
  void (*function)(int);                 /* thread body*/
  int arg;                                /* thread body function argument */
  bool active;
  long_t  WCET;
  long_t  period;
  long_t  deadline;
  long_t  phase;
  long_t  abs_deadline;         /* wall-clock time, not exec. cycles*/
  long_t  release;              /* wall-clock time, not exec. cycles*/
  long_t  next_release;         /* wall-clock time, not exec. cycles*/
  long_t  start;
  long_t  exec_cycles;
  long_t  WCEC;
  int nom_freq;                            /* nominal speed under S^can schedule for Aydin's DVS*/

  double U;
  double U_cc;
  double U_la;
  void* stack;

  long_t   slack;                 /*-- number of exec. cycles, NOT exec. time --*/
  long_t   Ca;                    /*-- number of exec. cycles, NOT exec. time --*/
  long_t   prevcc[CCWIN];         /*-- actual exec. cycle queue of previous jobs --*/
  int ccHead;                               /*-- prevcc queue index of the latest cc --*/  
  int canon_freq;
  int dStarted;
  float dC;                                 /* wall-clock time, not exec. cycles*/
  float dr;                                 /* wall-clock time, not exec. cycles*/

} thread;

/* the alpha Queue struct */
typedef struct node{
  int id;
  long_t  r; /* release time, wall-clock time */
  long_t  d; /* absolute deadline, wall-clock time */
  long_t  rem; /* remaining exec. time under S^can , NOT cycles ! */ 
  struct node * next;
} a_Q;



extern int fixedFreq;


extern int DVS;

extern long_t  hyperP;


extern void push2AlphaQ(a_Q ** head, int id, long_t  r, long_t  d, long_t  rem );


/* called from main() to start scheduling tasks infinitely */
extern void Start();

/* EDF scheduler */
extern void Scheduler(bool,int);

/*  Should be called before executing any of the other functions */
extern void initThreads();

/*  Creates a new thread, running the function that is passed as an argument. */
extern int spawnThread( void (*func)(void) , long_t WCET, long_t period, long_t deadline, long_t phase);

/*  Creates a new thread *func*, with function argument */
extern int spawnThreadArg( void (*func)(int) , int arg, long_t WCET, long_t period);


/* Create  *taskNum* number of threads, with a body function of *func*,  WCET utilization of *U*, and the average cc/WCET=*ccWC*.
 *base* is the basic timing multiplier factor */
extern int spawnMultiThd(  void (*func)(int) , int taskNum, float U, float ccWC, long_t base, unsigned int load);

/*  Yield control to another execution context */
extern void threadYield();

extern void my_dpm_init();

#ifdef SYNC
extern void my_set_freq(char* policy);
#else 
extern void my_set_freq(int task_state);
#endif 


#endif
